#!/usr/bin/env python3
# Adapted from Eli Bendersky's simple BF interpreter.

import sys      # for accessing command-line args

MEMORY_SIZE = 30000

# -----------------------------------------------------
# This functions parses a program string,
#      filtering out all non-command characters.
# -----------------------------------------------------

def parse_bf(src_str):
    program = ''
    for c in src_str:
        if c in '><+-.,[]':
            program += c
    return program

# -----------------------------------------------------
# The interpreter
# -----------------------------------------------------

def interpret(program):

    memory   = [0] * MEMORY_SIZE
    pc       = 0
    data_ptr = 0

    while pc < len(program):
        instruction = program[pc]

        if instruction == '>':
            data_ptr += 1
        elif instruction == '<':
            data_ptr -= 1
        elif instruction == '+':
            memory[data_ptr] += 1
        elif instruction == '-':
            memory[data_ptr] -= 1
        elif instruction == '.':
            print(chr(memory[data_ptr]), end='')
        elif instruction == ',':
            memory[data_ptr] = ord(sys.stdin.read(1))
        elif instruction == '[':
            if memory[data_ptr] == 0:
                bracket_nesting = 1;
                saved_pc = pc;                
                while bracket_nesting > 0 and pc < len(program) - 1:
                    pc += 1
                    if program[pc] == ']':
                        bracket_nesting -= 1
                    elif program[pc] == '[':
                        bracket_nesting += 1
                if bracket_nesting != 0:
                    raise ValueError('unmatched [ at pc = ', saved_pc)
        elif instruction == ']':
            if memory[data_ptr] != 0:
                bracket_nesting = 1;
                saved_pc = pc;
                while bracket_nesting > 0 and pc > 0:
                    pc -= 1
                    if program[pc] == '[':
                        bracket_nesting -= 1
                    elif program[pc] == ']':
                        bracket_nesting += 1
                if bracket_nesting != 0:
                    raise ValueError('unmatched ] at pc = ', saved_pc)
        else:
            msg = 'bad character {} at pc = {}'.format(instruction, pc)
            raise ValueError(msg)

        pc += 1

# -----------------------------------------------------
# The main function takes the command-line args as an
# array.  args[1] is the filename.
# -----------------------------------------------------

def main(args):
    bf_file_path = args[1]

    srcfile = open(bf_file_path, 'r')
    program = parse_bf(srcfile.read())
    
    interpret(program)

# -----------------------------------------------------
# These lines call the main function with one of three
# test programs, or with the command-line args
# -----------------------------------------------------

if __name__ == "__main__":
    # program_name = 'hello-world.bf'  # 'cell-size.bf' 'factor.bf'
    # main(['simple', program_name])
    main(sys.argv)
